@@ -55,11 +55,30 @@ class TestJunebugApi(JunebugTestBase):
             'The requested URL was not found on the server.  If you entered '
             'the URL manually please check your spelling and try again.', {
                 'errors': [{
+                    'code': 404,
                     'message': '404: Not Found',
                     'type': 'Not Found',
                     }]
                 })
 
+    @inlineCallbacks
+    def test_redirect_http_error(self):
+        resp = yield self.get('/channels')
+        [redirect] = resp.history()
+        yield self.assert_response(
+            redirect, http.MOVED_PERMANENTLY,
+            None, {
+                'errors': [{
+                    'code': 301,
+                    'message': '301: Moved Permanently',
+                    'type': 'Moved Permanently',
+                    'new_url': '%s/channels/' % self.url,
+                }],
+            })
+        yield self.assert_response(
+            resp, http.OK,
+            'channels listed', [])
+
     @inlineCallbacks
     def test_startup_plugins_started(self):
         '''When the API starts, all the configured plugins should start'''
@@ -204,10 +223,14 @@ class TestJunebugApi(JunebugTestBase):
                     {
                         'message': '-3 is less than the minimum of 0',
                         'type': 'invalid_body',
+                        'schema_path': [
+                            'properties', 'rate_limit_count', 'minimum'],
                     },
                     {
                         'message': "u'a' is not of type 'integer'",
                         'type': 'invalid_body',
+                        'schema_path': [
+                            'properties', 'character_limit', 'type'],
                     },
                 ])
             })
@@ -319,10 +342,14 @@ class TestJunebugApi(JunebugTestBase):
                     {
                         'message': '-3 is less than the minimum of 0',
                         'type': 'invalid_body',
+                        'schema_path': [
+                            'properties', 'rate_limit_count', 'minimum'],
                     },
                     {
                         'message': "u'a' is not of type 'integer'",
                         'type': 'invalid_body',
+                        'schema_path': [
+                            'properties', 'character_limit', 'type'],
                     },
                 ]
             })
@@ -359,6 +386,52 @@ class TestJunebugApi(JunebugTestBase):
         properties = yield self.redis.get('test-channel:properties')
         self.assertEqual(properties, None)
 
+    def record_channel_methods(self, *methods):
+        calls = []
+
+        def method_recorder(meth):
+            orig_method = getattr(Channel, meth)
+
+            def record(self, *args, **kw):
+                result = orig_method(self, *args, **kw)
+                calls.append((meth, self.id))
+                return result
+
+            return record
+
+        for meth in methods:
+            self.patch(Channel, meth, method_recorder(meth))
+        return calls
+
+    @inlineCallbacks
+    def test_restart_channel(self):
+        config = yield self.create_channel_config()
+        properties = self.create_channel_properties()
+        channel = Channel(self.redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+
+        actions = self.record_channel_methods('start', 'stop')
+
+        resp = yield self.post('/channels/test-channel/restart', None)
+        yield self.assert_response(resp, http.OK, 'channel restarted', {})
+
+        self.assertEqual(actions, [
+            ('stop', u'test-channel'),
+            ('start', u'test-channel'),
+        ])
+
+    @inlineCallbacks
+    def test_restart_missing_channel(self):
+        resp = yield self.post('/channels/test-channel/restart', None)
+        yield self.assert_response(
+            resp, http.NOT_FOUND, 'channel not found', {
+                'errors': [{
+                    'message': '',
+                    'type': 'ChannelNotFound',
+                }]
+            })
+
     @inlineCallbacks
     def test_send_message_invalid_channel(self):
         resp = yield self.post('/channels/foo-bar/messages/', {
@@ -510,6 +583,7 @@ class TestJunebugApi(JunebugTestBase):
                     'message': "Additional properties are not allowed (u'foo' "
                     "was unexpected)",
                     'type': 'invalid_body',
+                    'schema_path': ['additionalProperties'],
                 }]
             })
 
