@@ -26,7 +26,7 @@ class TestMessageForwardingWorker(JunebugTestBase):
         treq._utils.set_global_pool(connection_pool)
 
     @inlineCallbacks
-    def get_worker(self, config=None):
+    def get_worker(self, config=None, start=True):
         '''Get a new MessageForwardingWorker with the provided config'''
         if config is None:
             config = {}
@@ -47,7 +47,7 @@ class TestMessageForwardingWorker(JunebugTestBase):
             'metric_window': 1.0,
         }), config)
 
-        worker = yield self.app_helper.get_application(config)
+        worker = yield self.app_helper.get_application(config, start=start)
         returnValue(worker)
 
     @inlineCallbacks
@@ -135,6 +135,25 @@ class TestMessageForwardingWorker(JunebugTestBase):
             connector_name=worker.transport_name)
         self.assertEqual(dispatched_msg, msg)
 
+    @inlineCallbacks
+    def test_forward_event_no_message_id(self):
+        '''
+        If we receive an event, and we don't have a message ID to associate
+        the event to, then we should log the event and carry on.
+        '''
+        self.patch_logger()
+
+        event = TransportEvent(
+            event_type='ack',
+            user_message_id=None,
+            sent_message_id='msg-21',
+            timestamp='2015-09-22 15:39:44.827794')
+
+        yield self.worker.consume_ack(event)
+        self.assert_was_logged('Cannot store event')
+        self.assert_was_logged('Cannot find event URL')
+        self.assert_was_logged('%r' % event)
+
     @inlineCallbacks
     def test_forward_ack_http(self):
         event = TransportEvent(
@@ -508,6 +527,16 @@ class TestMessageForwardingWorker(JunebugTestBase):
         self.assertEqual((yield worker.message_rate.get_messages_per_second(
             'testtransport', 'delivery_pending', 1.0)), 1.0)
 
+    @inlineCallbacks
+    def test_teardown_without_startup(self):
+        '''If the teardown method is called before the worker was started up
+        correctly, the teardown should still succeed.'''
+        worker = yield self.get_worker(start=False)
+
+        self.assertEqual(getattr(worker, 'redis', None), None)
+
+        yield worker.teardown_application()
+
 
 class TestChannelStatusWorker(JunebugTestBase):
     @inlineCallbacks
